@@ -31,7 +31,6 @@
 #include "unity-shared/CairoTexture.h"
 #include "unity-shared/DashStyle.h"
 #include "unity-shared/TextureCache.h"
-#include "unity-shared/RawPixel.h"
 #include "unity-shared/UnitySettings.h"
 
 namespace unity
@@ -120,13 +119,7 @@ void ResultRendererTile::Render(nux::GraphicsEngine& GfxContext,
     return;
 
   dash::Style const& style = dash::Style::Instance();
-  RawPixel const tile_size   = style.GetTileImageSize();
-  RawPixel const tile_width  = style.GetTileWidth();
-  RawPixel const tile_height = style.GetTileHeight();
-  RawPixel const tile_highlight_width  = style.GetTileIconHightlightWidth();
-  RawPixel const tile_highlight_height = style.GetTileIconHightlightHeight();
-
-  int tile_icon_size = tile_size.CP(scale());
+  int tile_icon_size = style.GetTileImageSize().CP(scale);
 
   // set up our texture mode
   nux::TexCoordXForm texxform;
@@ -148,8 +141,8 @@ void ResultRendererTile::Render(nux::GraphicsEngine& GfxContext,
   // render highlight if its needed
   if (container->prelight && state != ResultRendererState::RESULT_RENDERER_NORMAL)
   {
-    int highlight_x =  (geometry.x + geometry.width/2) - tile_highlight_width.CP(scale())/2;
-    int highlight_y =  (geometry.y + PADDING.CP(scale()) + tile_icon_size / 2) - tile_highlight_height.CP(scale())/2;
+    int highlight_x = (geometry.x + geometry.width/2) - style.GetTileIconHightlightWidth().CP(scale)/2;
+    int highlight_y = (geometry.y + PADDING.CP(scale) + tile_icon_size / 2) - style.GetTileIconHightlightHeight().CP(scale)/2;
 
     RenderTexture(GfxContext,
                   highlight_x,
@@ -179,10 +172,10 @@ void ResultRendererTile::Render(nux::GraphicsEngine& GfxContext,
   if (container->text)
   {
     RenderTexture(GfxContext,
-                  geometry.x + PADDING.CP(scale()),
-                  geometry.y + tile_icon_size + SPACING.CP(scale()),
-                  tile_width.CP(scale()) - (PADDING.CP(scale()) * 2),
-                  tile_height.CP(scale()) - tile_icon_size - SPACING.CP(scale()),
+                  geometry.x + PADDING.CP(scale),
+                  geometry.y + tile_icon_size + SPACING.CP(scale),
+                  style.GetTileWidth().CP(scale) - (PADDING.CP(scale) * 2),
+                  style.GetTileHeight().CP(scale) - tile_icon_size - SPACING.CP(scale),
                   container->text->GetDeviceTexture(),
                   texxform,
                   color,
@@ -273,10 +266,8 @@ nux::NBitmapData* ResultRendererTile::GetDndImage(Result const& row) const
 void ResultRendererTile::LoadIcon(Result const& row)
 {
   Style const& style = Style::Instance();
-  RawPixel const tile_size   = style.GetTileImageSize();
-  RawPixel const tile_gsize  = style.GetTileGIconSize();
-  RawPixel const tile_highlight_width  = style.GetTileIconHightlightWidth();
-  RawPixel const tile_highlight_height = style.GetTileIconHightlightHeight();
+  int tile_size   = style.GetTileImageSize().CP(scale);
+  int tile_gsize  = style.GetTileGIconSize().CP(scale);
 
   std::string const& icon_hint = row.icon_hint;
   std::string const& icon_name = !icon_hint.empty() ? icon_hint : DEFAULT_GICON;
@@ -288,8 +279,8 @@ void ResultRendererTile::LoadIcon(Result const& row)
   {
     TextureCache& cache = TextureCache::GetDefault();
     BaseTexturePtr texture_prelight(cache.FindTexture("resultview_prelight",
-                                                      tile_highlight_width.CP(scale()),
-                                                      tile_highlight_height.CP(scale()),
+                                                      style.GetTileIconHightlightWidth().CP(scale),
+                                                      style.GetTileIconHightlightHeight().CP(scale),
                                                       sigc::mem_fun(this, &ResultRendererTile::DrawHighlight)));
     container->prelight = texture_prelight;
   }
@@ -300,13 +291,13 @@ void ResultRendererTile::LoadIcon(Result const& row)
   {
     bool use_large_icon = icon.IsType(G_TYPE_FILE_ICON) || !icon.IsType(G_TYPE_THEMED_ICON);
     container->slot_handle = IconLoader::GetDefault().LoadFromGIconString(icon_name, 
-                                                                          tile_size.CP(scale()),
+                                                                          tile_size,
                                                                           use_large_icon ?
-                                                                          tile_size.CP(scale()) : tile_gsize.CP(scale()), slot);
+                                                                          tile_size : tile_gsize, slot);
   }
   else
   {
-    container->slot_handle = IconLoader::GetDefault().LoadFromIconName(icon_name, -1, tile_gsize.CP(scale()), slot);
+    container->slot_handle = IconLoader::GetDefault().LoadFromIconName(icon_name, -1, tile_gsize, slot);
   }
 }
 
@@ -333,15 +324,12 @@ nux::BaseTexture* ResultRendererTile::CreateTextureCallback(std::string const& t
   }
   else
   {
-    Style const& style = Style::Instance();
-    RawPixel const tile_size = style.GetTileImageSize();
-
     // slow path for non square icons that must be resized to fit in the square
     // texture
     float aspect = static_cast<float>(pixbuf_height) / pixbuf_width; // already sanitized width/height so can not be 0.0
     if (aspect < 1.0f)
     {
-      pixbuf_width = tile_size.CP(scale());
+      pixbuf_width = Style::Instance().GetTileImageSize().CP(scale);
       pixbuf_height = pixbuf_width * aspect;
 
       if (pixbuf_height > height)
@@ -470,13 +458,10 @@ std::string ReplaceBlacklistedChars(std::string const& str)
 void ResultRendererTile::LoadText(Result const& row)
 {
   Style const& style = Style::Instance();
-  RawPixel const tile_size   = style.GetTileImageSize();
-  RawPixel const tile_width  = style.GetTileWidth();
-  RawPixel const tile_height = style.GetTileHeight();
 
   nux::CairoGraphics _cairoGraphics(CAIRO_FORMAT_ARGB32,
-                                    tile_width.CP(scale()) - (PADDING.CP(scale()) * 2),
-                                    tile_height.CP(scale()) - tile_size.CP(scale()) - SPACING.CP(scale()));
+                                    style.GetTileWidth().CP(scale()) - (PADDING.CP(scale()) * 2),
+                                    style.GetTileHeight().CP(scale()) - style.GetTileImageSize().CP(scale()) - SPACING.CP(scale()));
   cairo_surface_set_device_scale(_cairoGraphics.GetSurface(), scale(), scale());
 
   cairo_t* cr = _cairoGraphics.GetInternalContext();
@@ -499,7 +484,7 @@ void ResultRendererTile::LoadText(Result const& row)
 
   pango_layout_set_wrap(layout, PANGO_WRAP_WORD_CHAR);
   pango_layout_set_ellipsize(layout, PANGO_ELLIPSIZE_START);
-  pango_layout_set_width(layout, (tile_width - (PADDING * 2))* PANGO_SCALE);
+  pango_layout_set_width(layout, (style.GetTileWidth() - (PADDING * 2))* PANGO_SCALE);
   pango_layout_set_height(layout, -2);
 
   // FIXME bug #1239381
