@@ -29,9 +29,8 @@
 #include "ResultRendererHorizontalTile.h"
 #include "unity-shared/UBusMessages.h"
 #include "unity-shared/UBusWrapper.h"
-#include "unity-shared/PlacesOverlayVScrollBar.h"
+#include "unity-shared/OverlayScrollView.h"
 #include "unity-shared/GraphicsUtils.h"
-#include "unity-shared/RawPixel.h"
 
 #include "config.h"
 #include <glib/gi18n-lib.h>
@@ -52,16 +51,14 @@ const double DEFAULT_SCALE         = 1.0;
 }
 
 // This is so we can access some protected members in scrollview.
-class ScopeScrollView: public nux::ScrollView
+class ScopeScrollView: public dash::ScrollView
 {
 public:
-  ScopeScrollView(nux::VScrollBar* scroll_bar, NUX_FILE_LINE_DECL)
-    : nux::ScrollView(NUX_FILE_LINE_PARAM)
+  ScopeScrollView(NUX_FILE_LINE_DECL)
+    : ScrollView(NUX_FILE_LINE_PARAM)
     , right_area_(nullptr)
     , up_area_(nullptr)
   {
-    SetVScrollBar(scroll_bar);
-
     OnVisibleChanged.connect([this] (nux::Area* /*area*/, bool visible) {
       if (m_horizontal_scrollbar_enable)
         _hscrollbar->SetVisible(visible);
@@ -237,8 +234,8 @@ void ScopeView::SetupViews(nux::Area* show_filters)
 {
   layout_ = new nux::HLayout(NUX_TRACKER_LOCATION);
 
-  scroll_view_ = new ScopeScrollView(new PlacesOverlayVScrollBar(NUX_TRACKER_LOCATION),
-                                    NUX_TRACKER_LOCATION);
+  scroll_view_ = new ScopeScrollView(NUX_TRACKER_LOCATION);
+  scroll_view_->scale = scale();
   scroll_view_->EnableVerticalScrollBar(true);
   scroll_view_->EnableHorizontalScrollBar(false);
   layout_->AddView(scroll_view_);
@@ -250,9 +247,11 @@ void ScopeView::SetupViews(nux::Area* show_filters)
   no_results_ = new StaticCairoText("", NUX_TRACKER_LOCATION);
   no_results_->SetTextColor(nux::color::White);
   no_results_->SetVisible(false);
+  no_results_->SetScale(scale);
   scroll_layout_->AddView(no_results_, 1, nux::MINOR_POSITION_CENTER, nux::MINOR_SIZE_MATCHCONTENT);
 
-  fscroll_view_ = new ScopeScrollView(new PlacesOverlayVScrollBar(NUX_TRACKER_LOCATION), NUX_TRACKER_LOCATION);
+  fscroll_view_ = new ScopeScrollView(NUX_TRACKER_LOCATION);
+  fscroll_view_->scale = scale();
   fscroll_view_->EnableVerticalScrollBar(true);
   fscroll_view_->EnableHorizontalScrollBar(false);
   fscroll_view_->SetVisible(false);
@@ -276,19 +275,17 @@ void ScopeView::UpdateScopeViewSize()
 {
   dash::Style const& style = dash::Style::Instance();
 
-  RawPixel const scope_filter_space = style.GetSpaceBetweenScopeAndFilters();
-  RawPixel const right_padding      = style.GetFilterViewRightPadding();
-  RawPixel const filter_width       = style.GetFilterBarWidth() +
-                                      style.GetFilterBarLeftPadding() +
-                                      style.GetFilterBarRightPadding();
+  int right_padding = style.GetFilterViewRightPadding().CP(scale);
+  int filter_width  = style.GetFilterBarWidth().CP(scale) +
+                      style.GetFilterBarLeftPadding().CP(scale) +
+                      style.GetFilterBarRightPadding().CP(scale);
 
-  double scale = this->scale();
-  layout_->SetSpaceBetweenChildren(scope_filter_space.CP(scale));
+  layout_->SetSpaceBetweenChildren(style.GetSpaceBetweenScopeAndFilters().CP(scale));
 
-  fscroll_view_->SetMinimumWidth(filter_width.CP(scale) + right_padding.CP(scale));
-  fscroll_view_->SetMaximumWidth(filter_width.CP(scale) + right_padding.CP(scale));
-  filter_bar_->SetMinimumWidth(filter_width.CP(scale));
-  filter_bar_->SetMaximumWidth(filter_width.CP(scale));
+  fscroll_view_->SetMinimumWidth(filter_width + right_padding);
+  fscroll_view_->SetMaximumWidth(filter_width + right_padding);
+  filter_bar_->SetMinimumWidth(filter_width);
+  filter_bar_->SetMaximumWidth(filter_width);
 }
 
 void ScopeView::UpdateScale(double scale)
@@ -298,7 +295,10 @@ void ScopeView::UpdateScale(double scale)
   for (auto& group : category_views_)
     group->scale = scale;
 
+  scroll_view_->scale = scale;
+  fscroll_view_->scale = scale;
   filter_bar_->scale = scale;
+  no_results_->SetScale(scale);
 }
 
 void ScopeView::SetupCategories(Categories::Ptr const& categories)
