@@ -45,9 +45,9 @@ public:
   bool AcceptKeyNavFocus() override { return true; }
   bool InspectKeyEvent(unsigned int, unsigned int, const char*) override { return true; };
 };
-}
 
 DECLARE_LOGGER(logger, "unity.lockscreen");
+}
 
 Controller::Controller(DBusManager::Ptr const& dbus_manager,
                        session::Manager::Ptr const& session_manager,
@@ -100,11 +100,10 @@ Controller::Controller(DBusManager::Ptr const& dbus_manager,
   fade_animator_.finished.connect([this] {
     if (animation::GetDirection(fade_animator_) == animation::Direction::BACKWARD)
     {
-      motion_connection_->disconnect();
-      key_connection_->disconnect();
+      primary_shield_connections_.Clear();
       uscreen_connection_->block();
       hidden_window_connection_->block();
-      session_manager_->unlocked.emit();
+      session_manager_->is_locked = false;
 
       std::for_each(shields_.begin(), shields_.end(), [](nux::ObjectPtr<Shield> const& shield) {
         shield->RemoveLayout();
@@ -173,10 +172,7 @@ void Controller::OnPrimaryShieldMotion(int x, int y)
       primary_shield_ = shield;
       shield->primary = true;
       nux::GetWindowCompositor().SetAlwaysOnFrontWindow(primary_shield_.GetPointer());
-      auto move_cb = sigc::mem_fun(this, &Controller::OnPrimaryShieldMotion);
-      motion_connection_ = shield->grab_motion.connect(move_cb);
-      auto key_cb = sigc::hide(sigc::hide(sigc::mem_fun(this, &Controller::ResetPostLockScreenSaver)));
-      key_connection_ = shield->grab_key.connect(key_cb);
+      SetupPrimaryShieldConnections();
       break;
     }
   }
@@ -184,14 +180,52 @@ void Controller::OnPrimaryShieldMotion(int x, int y)
   ResetPostLockScreenSaver();
 }
 
+void Controller::SetupPrimaryShieldConnections()
+{
+  if (!primary_shield_.IsValid())
+    return;
+
+  primary_shield_connections_.Clear();
+
+  auto move_cb = sigc::mem_fun(this, &Controller::OnPrimaryShieldMotion);
+  primary_shield_connections_.Add(primary_shield_->grab_motion.connect(move_cb));
+
+  auto key_cb = sigc::hide(sigc::hide(sigc::mem_fun(this, &Controller::ResetPostLockScreenSaver)));
+  primary_shield_connections_.Add(primary_shield_->grab_key.connect(key_cb));
+
+  if (!session_manager_->is_locked())
+  {
+    primary_shield_connections_.Add(primary_shield_->grabbed.connect([this] {
+      session_manager_->is_locked = true;
+    }));
+
+    primary_shield_connections_.Add(primary_shield_->grab_failed.connect([this] {
+      if (!session_manager_->is_locked())
+      {
+        LOG_ERROR(logger) << "Impossible to get the grab to lock the screen";
+        session_manager_->unlock_requested.emit();
+      }
+    }));
+  }
+}
+
 void Controller::EnsureShields(std::vector<nux::Geometry> const& monitors)
 {
   int num_monitors = monitors.size();
   int shields_size = shields_.size();
   int primary = UScreen::GetDefault()->GetMonitorWithMouse();
 
+  // Keep a reference of the old prompt_view
+  nux::ObjectPtr<UserPromptView> prompt_view(prompt_view_.GetPointer());
+
   shields_.resize(num_monitors);
 
+  if (!prompt_view)
+  {
+    prompt_view = test_mode_ ? nullptr : new UserPromptView(session_manager_);
+    prompt_view_ = prompt_view.GetPointer();
+  }
+
   for (int i = 0; i < num_monitors; ++i)
   {
     auto& shield = shields_[i];
@@ -199,15 +233,22 @@ void Controller::EnsureShields(std::vector<nux::Geometry> const& monitors)
 
     if (i >= shields_size)
     {
-      shield = shield_factory_->CreateShield(session_manager_, indicators_, accelerator_controller_->GetAccelerators(), i, i == primary);
+      shield = shield_factory_->CreateShield(session_manager_, indicators_, accelerator_controller_->GetAccelerators(), prompt_view, i, i == primary);
       is_new = true;
     }
 
-    shield->SetGeometry(monitors[i]);
-    shield->SetMinMaxSize(monitors[i].width, monitors[i].height);
-    shield->primary = (i == primary);
+    auto old_geo = shield->GetGeometry();
+    auto new_geo = monitors[i];
+
+    shield->SetGeometry(new_geo);
+    shield->SetMinMaxSize(new_geo.width, new_geo.height);
+    shield->primary = false;
     shield->monitor = i;
 
+    // XXX: manually emit nux::Area::geometry_changed beucase nux can fail to emit it.
+    if (old_geo != new_geo)
+      shield->geometry_changed.emit(shield.GetPointer(), new_geo);
+
     if (is_new && fade_animator_.GetCurrentValue() > 0)
     {
       shield->SetOpacity(fade_animator_.GetCurrentValue());
@@ -217,10 +258,7 @@ void Controller::EnsureShields(std::vector<nux::Geometry> const& monitors)
 
   primary_shield_ = shields_[primary];
   primary_shield_->primary = true;
-  auto move_cb = sigc::mem_fun(this, &Controller::OnPrimaryShieldMotion);
-  motion_connection_ = primary_shield_->grab_motion.connect(move_cb);
-  auto key_cb = sigc::hide(sigc::hide(sigc::mem_fun(this, &Controller::ResetPostLockScreenSaver)));
-  key_connection_ = primary_shield_->grab_key.connect(key_cb);
+  SetupPrimaryShieldConnections();
 }
 
 void Controller::EnsureBlankWindow()
@@ -342,7 +380,6 @@ void Controller::OnLockRequested(bool prompt)
       HideBlankWindow();
 
     LockScreen();
-    session_manager_->locked.emit();
 
     if (prompt_activation_)
     {
@@ -403,7 +440,7 @@ void Controller::LockScreen()
   indicators_ = std::make_shared<indicator::LockScreenDBusIndicators>();
   upstart_wrapper_->Emit("desktop-lock");
 
-  accelerator_controller_ = std::make_shared<AcceleratorController>();
+  accelerator_controller_ = std::make_shared<AcceleratorController>(session_manager_);
   auto activate_key = WindowManager::Default().activate_indicators_key();
   auto accelerator = std::make_shared<Accelerator>(activate_key.second, 0, activate_key.first);
   accelerator->activated.connect(std::bind(std::mem_fn(&Controller::ActivatePanel), this));
@@ -428,6 +465,7 @@ void Controller::ShowShields()
     shield->PushToFront();
   });
 
+  session_manager_->is_locked = primary_shield_->HasGrab();
   nux::GetWindowCompositor().SetAlwaysOnFrontWindow(primary_shield_.GetPointer());
   animation::StartOrReverse(fade_animator_, animation::Direction::FORWARD);
 }
