@@ -27,6 +27,7 @@
 #include "LockScreenShieldFactory.h"
 #include "LockScreenAcceleratorController.h"
 #include "ScreenSaverDBusManager.h"
+#include "UserPromptView.h"
 #include "unity-shared/BackgroundEffectHelper.h"
 #include "unity-shared/UpstartWrapper.h"
 
@@ -35,9 +36,13 @@ namespace unity
 namespace lockscreen
 {
 
+class UserPromptView;
+
 class Controller : public sigc::trackable
 {
 public:
+  typedef std::shared_ptr<Controller> Ptr;
+
   Controller(DBusManager::Ptr const&, session::Manager::Ptr const&,
              UpstartWrapper::Ptr const& upstart_wrapper = std::make_shared<UpstartWrapper>(),
              ShieldFactoryInterface::Ptr const& shield_factory = std::make_shared<ShieldFactory>(),
@@ -61,6 +66,7 @@ private:
   void BlankWindowGrabEnable(bool grab);
   void SimulateActivity();
   void ResetPostLockScreenSaver();
+  void SetupPrimaryShieldConnections();
   void ActivatePanel();
 
   void OnLockRequested(bool prompt);
@@ -71,6 +77,7 @@ private:
 
   std::vector<nux::ObjectPtr<AbstractShield>> shields_;
   nux::ObjectWeakPtr<AbstractShield> primary_shield_;
+  nux::ObjectWeakPtr<UserPromptView> prompt_view_;
   nux::ObjectPtr<nux::BaseWindow> blank_window_;
 
   DBusManager::Ptr dbus_manager_;
@@ -90,8 +97,7 @@ private:
   connection::Wrapper uscreen_connection_;
   connection::Wrapper suspend_connection_;
   connection::Wrapper hidden_window_connection_;
-  connection::Wrapper motion_connection_;
-  connection::Wrapper key_connection_;
+  connection::Manager primary_shield_connections_;
 
   glib::Source::UniquePtr lockscreen_timeout_;
   glib::Source::UniquePtr lockscreen_delay_timeout_;
