@@ -20,6 +20,7 @@
 
 #include <Nux/Nux.h>
 #include <NuxCore/Logger.h>
+#include <boost/algorithm/string/erase.hpp>
 
 #include "PanelMenuView.h"
 #include "unity-shared/AnimationUtils.h"
@@ -53,6 +54,32 @@ namespace
   const std::string WINDOW_ACTIVATED_TIMEOUT = "window-activated-timeout";
   const std::string UPDATE_SHOW_NOW_TIMEOUT = "update-show-now-timeout";
   const std::string INTEGRATED_MENUS_DOUBLE_CLICK_TIMEOUT = "integrated-menus-double-click-timeout";
+
+std::string get_current_desktop()
+{
+  std::ifstream fin("/etc/os-release");
+  std::string temp;
+  std::string os_release_name("Ubuntu");
+
+  if (fin.is_open())
+  {
+    while (getline(fin, temp))
+    {
+      if (temp.substr(0,4) == "NAME")
+      {
+        os_release_name = boost::erase_all_copy(temp.substr(temp.find_last_of('=')+1), "\"");
+        break;
+      }
+    }
+    fin.close();
+  }
+
+  //this is done to avoid breaking translation before 14.10.
+  if (os_release_name.empty() || os_release_name == "Ubuntu")
+    return _("Ubuntu Desktop");
+  else
+    return glib::String(g_strdup_printf(_("%s Desktop"), os_release_name.c_str())).Str();
+}
 }
 
 PanelMenuView::PanelMenuView(menu::Manager::Ptr const& menus)
@@ -73,9 +100,8 @@ PanelMenuView::PanelMenuView(menu::Manager::Ptr const& menus)
   , ignore_menu_visibility_(false)
   , integrated_menus_(decoration::Style::Get()->integrated_menus())
   , active_xid_(0)
-  , desktop_name_(_("Ubuntu Desktop"))
+  , desktop_name_(get_current_desktop())
 {
-
   BamfWindow* active_win = bamf_matcher_get_active_window(matcher_);
   if (BAMF_IS_WINDOW(active_win))
     active_xid_ = bamf_window_get_xid(active_win);
