@@ -46,6 +46,9 @@ G_DEFINE_TYPE (PanelService, panel_service, G_TYPE_OBJECT);
 #define N_TIMEOUT_SLOTS 50
 #define MAX_INDICATOR_ENTRIES 100
 
+#define NUX_VERTICAL_SCROLL_DELTA 120
+#define NUX_HORIZONTAL_SCROLL_DELTA (NUX_VERTICAL_SCROLL_DELTA ^ 2)
+
 #define COMPIZ_OPTION_SCHEMA "org.compiz.unityshell"
 #define COMPIZ_OPTION_PATH "/org/compiz/profiles/unity/plugins/"
 #define MENU_TOGGLE_KEYBINDING_KEY "panel-first-menu"
@@ -276,6 +279,15 @@ panel_service_class_init (PanelServiceClass *klass)
   g_type_class_add_private (obj_class, sizeof (PanelServicePrivate));
 }
 
+static gboolean
+is_point_in_rect (gint x, gint y, GdkRectangle* rect)
+{
+  g_return_val_if_fail (rect, FALSE);
+
+  return (x >= rect->x && x <= (rect->x + rect->width) &&
+          y >= rect->y && y <= (rect->y + rect->height));
+}
+
 IndicatorObjectEntry *
 get_entry_at (PanelService *self, gint x, gint y)
 {
@@ -293,8 +305,7 @@ get_entry_at (PanelService *self, gint x, gint y)
           IndicatorObjectEntry *entry = k;
           GdkRectangle *geo = v;
 
-          if (x >= geo->x && x <= (geo->x + geo->width) &&
-              y >= geo->y && y <= (geo->y + geo->height))
+          if (is_point_in_rect (x, y, geo))
             {
               return entry;
             }
@@ -1623,12 +1634,42 @@ indicator_object_to_variant (IndicatorObject *object, const gchar *indicator_id,
     }
 }
 
+static int
+get_monitor_at (gint x, gint y)
+{
+  gint i;
+  gdouble scale;
+  GdkScreen *screen = gdk_screen_get_default ();
+  gint monitors = gdk_screen_get_n_monitors (screen);
+
+  for (i = 0; i < monitors; ++i)
+    {
+      GdkRectangle rect = { 0 };
+      gdk_screen_get_monitor_geometry (screen, i, &rect);
+      scale = gdk_screen_get_monitor_scale_factor (screen, i);
+
+      if (scale != 1.0)
+        {
+          rect.x *= scale;
+          rect.y *= scale;
+          rect.width *= scale;
+          rect.height *= scale;
+        }
+
+      if (is_point_in_rect (x, y, &rect))
+        {
+          return i;
+        }
+    }
+
+  return gdk_screen_get_monitor_at_point (screen, x, y);
+}
+
 static int
 get_monitor_scale_at (gint x, gint y)
 {
-  GdkScreen *screen = gdk_screen_get_default ();
-  int monitor = gdk_screen_get_monitor_at_point (screen, x, y);
-  return gdk_screen_get_monitor_scale_factor (screen, monitor);
+  gint monitor = get_monitor_at (x, y);
+  return gdk_screen_get_monitor_scale_factor (gdk_screen_get_default (), monitor);
 }
 
 static void
@@ -1642,7 +1683,7 @@ positon_menu (GtkMenu  *menu,
   PanelServicePrivate *priv = self->priv;
 
   GdkScreen *screen = gdk_screen_get_default ();
-  gint monitor = gdk_screen_get_monitor_at_point (screen, priv->last_x, priv->last_y);
+  gint monitor = get_monitor_at (priv->last_x, priv->last_y);
   gtk_menu_set_monitor (menu, monitor);
 
   gint scale = gdk_screen_get_monitor_scale_factor (screen, monitor);
@@ -2159,12 +2200,24 @@ panel_service_show_entry_common (PanelService *self,
       g_signal_connect_after (priv->last_menu, "move-current",
                               G_CALLBACK (on_active_menu_move_current), self);
 
-      gtk_menu_popup (priv->last_menu, NULL, NULL, positon_menu, self, 0, CurrentTime);
-      gtk_menu_reposition (priv->last_menu);
+      gtk_menu_shell_set_take_focus (GTK_MENU_SHELL (priv->last_menu), TRUE);
+      gtk_menu_popup (priv->last_menu, NULL, NULL, positon_menu, self, button, CurrentTime);
+      gboolean visible = gtk_widget_is_visible (GTK_WIDGET (priv->last_menu));
 
-      GdkWindow *gdkwin = gtk_widget_get_window (GTK_WIDGET (priv->last_menu));
-      if (gdkwin != NULL)
+      if (!visible)
         {
+          /* If the menu is not visible at this point, it's very likely that's
+           * due to a keyboard grab, so let's try with a menu with no key-grab */
+          gtk_menu_shell_set_take_focus (GTK_MENU_SHELL (priv->last_menu), FALSE);
+          gtk_menu_popup (priv->last_menu, NULL, NULL, positon_menu, self, button, CurrentTime);
+          visible = gtk_widget_is_visible (GTK_WIDGET (priv->last_menu));
+        }
+
+      if (visible)
+        {
+          gtk_menu_reposition (priv->last_menu);
+
+          GdkWindow *gdkwin = gtk_widget_get_window (GTK_WIDGET (priv->last_menu));
           gint left=0, top=0, width=0, height=0;
 
           gdk_window_get_geometry (gdkwin, NULL, NULL, &width, &height);
@@ -2182,10 +2235,7 @@ panel_service_show_entry_common (PanelService *self,
         }
       else
         {
-          priv->last_left = 0;
-          priv->last_right = 0;
-          priv->last_top = 0;
-          priv->last_bottom = 0;
+          on_active_menu_hidden (priv->last_menu, self);
         }
     }
 
@@ -2371,6 +2421,8 @@ panel_service_secondary_activate_entry (PanelService *self, const gchar *entry_i
   IndicatorObject      *object;
   IndicatorObjectEntry *entry;
 
+  g_return_if_fail (PANEL_IS_SERVICE (self));
+
   entry = get_indicator_entry_by_id (self, entry_id);
   g_return_if_fail (entry);
 
@@ -2387,12 +2439,44 @@ panel_service_scroll_entry (PanelService   *self,
   IndicatorObject      *object;
   IndicatorObjectEntry *entry;
 
+  g_return_if_fail (PANEL_IS_SERVICE (self));
+
   entry = get_indicator_entry_by_id (self, entry_id);
   g_return_if_fail (entry);
 
-  GdkScrollDirection direction = delta < 0 ? GDK_SCROLL_DOWN : GDK_SCROLL_UP;
+  GdkScrollDirection direction = G_MAXINT;
 
-  object = get_entry_parent_indicator (entry);
-  g_signal_emit_by_name(object, INDICATOR_OBJECT_SIGNAL_ENTRY_SCROLLED, entry,
-                        abs(delta/120), direction);
+  switch (delta)
+    {
+      case NUX_VERTICAL_SCROLL_DELTA:
+        direction = INDICATOR_OBJECT_SCROLL_UP;
+        break;
+      case -NUX_VERTICAL_SCROLL_DELTA:
+        direction = INDICATOR_OBJECT_SCROLL_DOWN;
+        break;
+      case NUX_HORIZONTAL_SCROLL_DELTA:
+        direction = INDICATOR_OBJECT_SCROLL_LEFT;
+        break;
+      case -NUX_HORIZONTAL_SCROLL_DELTA:
+        direction = INDICATOR_OBJECT_SCROLL_RIGHT;
+        break;
+      }
+
+  if (direction != G_MAXINT)
+    {
+      object = get_entry_parent_indicator (entry);
+      g_signal_emit_by_name(object, INDICATOR_OBJECT_SIGNAL_ENTRY_SCROLLED,
+                            entry, 1, direction);
+    }
+}
+
+void
+panel_service_close_active_entry (PanelService *self)
+{
+  g_return_if_fail (PANEL_IS_SERVICE (self));
+
+  if (GTK_IS_MENU (self->priv->last_menu))
+    {
+      gtk_menu_popdown (GTK_MENU (self->priv->last_menu));
+    }
 }
